@@ -1,7 +1,9 @@
 #[cfg(any(feature = "v2016_4", feature = "dox"))]
 use crate::RepoListRefsExtFlags;
-use crate::{Checksum, ObjectName, ObjectType, Repo, RepoTransactionStats};
-use ffi;
+#[cfg(feature = "cap-std-apis")]
+use crate::RepoMode;
+use crate::{Checksum, ObjectDetails, ObjectName, ObjectType, Repo, RepoTransactionStats};
+use ffi::OstreeRepoListObjectsFlags;
 use glib::ffi as glib_sys;
 use glib::{self, translate::*, Error, IsA};
 use std::{
@@ -23,6 +25,20 @@ unsafe extern "C" fn read_variant_table(
     set.insert(ObjectName::new_from_variant(value));
 }
 
+unsafe extern "C" fn read_variant_object_map(
+    key: glib_sys::gpointer,
+    value: glib_sys::gpointer,
+    hash_set: glib_sys::gpointer,
+) {
+    let key: glib::Variant = from_glib_none(key as *const glib_sys::GVariant);
+    let value: glib::Variant = from_glib_none(value as *const glib_sys::GVariant);
+    let set: &mut HashMap<ObjectName, ObjectDetails> =
+        &mut *(hash_set as *mut HashMap<ObjectName, ObjectDetails>);
+    if let Some(details) = ObjectDetails::new_from_variant(value) {
+        set.insert(ObjectName::new_from_variant(key), details);
+    }
+}
+
 unsafe fn from_glib_container_variant_set(ptr: *mut glib_sys::GHashTable) -> HashSet<ObjectName> {
     let mut set = HashSet::new();
     glib_sys::g_hash_table_foreach(
@@ -34,6 +50,19 @@ unsafe fn from_glib_container_variant_set(ptr: *mut glib_sys::GHashTable) -> Has
     set
 }
 
+unsafe fn from_glib_container_variant_map(
+    ptr: *mut glib_sys::GHashTable,
+) -> HashMap<ObjectName, ObjectDetails> {
+    let mut set = HashMap::new();
+    glib_sys::g_hash_table_foreach(
+        ptr,
+        Some(read_variant_object_map),
+        &mut set as *mut HashMap<ObjectName, ObjectDetails> as *mut _,
+    );
+    glib_sys::g_hash_table_unref(ptr);
+    set
+}
+
 /// An open transaction in the repository.
 ///
 /// This will automatically invoke [`ostree::Repo::abort_transaction`] when the value is dropped.
@@ -70,6 +99,26 @@ impl Repo {
         Repo::new(&gio::File::for_path(path.as_ref()))
     }
 
+    #[cfg(feature = "cap-std-apis")]
+    /// A version of [`open_at`] which uses cap-std.
+    pub fn open_at_dir(dir: &cap_std::fs::Dir, path: &str) -> Result<Repo, glib::Error> {
+        use std::os::unix::io::AsRawFd;
+        crate::Repo::open_at(dir.as_raw_fd(), path, gio::NONE_CANCELLABLE)
+    }
+
+    #[cfg(feature = "cap-std-apis")]
+    /// A version of [`create_at`] which uses cap-std, and also returns the opened repo.
+    pub fn create_at_dir(
+        dir: &cap_std::fs::Dir,
+        path: &str,
+        mode: RepoMode,
+        options: Option<&glib::Variant>,
+    ) -> Result<Repo, glib::Error> {
+        use std::os::unix::io::AsRawFd;
+        crate::Repo::create_at(dir.as_raw_fd(), path, mode, options, gio::NONE_CANCELLABLE)?;
+        Repo::open_at_dir(dir, path)
+    }
+
     /// A wrapper for [`prepare_transaction`] which ensures the transaction will be aborted when the guard goes out of scope.
     pub fn auto_transaction<P: IsA<gio::Cancellable>>(
         &self,
@@ -92,10 +141,22 @@ impl Repo {
             let copy = dfd.try_clone();
             // Now release our temporary ownership of the original
             let _ = dfd.into_raw_fd();
-            Ok(copy?)
+            copy
         }
     }
 
+    /// Borrow the directory file descriptor for this repository.
+    #[cfg(feature = "cap-std-apis")]
+    pub fn dfd_borrow<'a>(&'a self) -> io_lifetimes::BorrowedFd<'a> {
+        unsafe { io_lifetimes::BorrowedFd::borrow_raw_fd(self.dfd()) }
+    }
+
+    /// Return a new `cap-std` directory reference for this repository.
+    #[cfg(feature = "cap-std-apis")]
+    pub fn dfd_as_dir(&self) -> std::io::Result<cap_std::fs::Dir> {
+        cap_std::fs::Dir::reopen_dir(&self.dfd_borrow())
+    }
+
     /// Find all objects reachable from a commit.
     pub fn traverse_commit<P: IsA<gio::Cancellable>>(
         &self,
@@ -147,6 +208,32 @@ impl Repo {
         }
     }
 
+    /// List all repo objects
+    pub fn list_objects<P: IsA<gio::Cancellable>>(
+        &self,
+        flags: OstreeRepoListObjectsFlags,
+        cancellable: Option<&P>,
+    ) -> Result<HashMap<ObjectName, ObjectDetails>, Error> {
+        unsafe {
+            let mut error = ptr::null_mut();
+            let mut hashtable = ptr::null_mut();
+
+            ffi::ostree_repo_list_objects(
+                self.to_glib_none().0,
+                flags,
+                &mut hashtable,
+                cancellable.map(AsRef::as_ref).to_glib_none().0,
+                &mut error,
+            );
+
+            if error.is_null() {
+                Ok(from_glib_container_variant_map(hashtable))
+            } else {
+                Err(from_glib_full(error))
+            }
+        }
+    }
+
     /// List refs with extended options.
     #[cfg(any(feature = "v2016_4", feature = "dox"))]
     pub fn list_refs_ext<P: IsA<gio::Cancellable>>(
@@ -175,6 +262,13 @@ impl Repo {
         }
     }
 
+    /// Resolve a refspec to a commit SHA256.
+    /// Returns an error if the refspec does not exist.
+    pub fn require_rev(&self, refspec: &str) -> Result<glib::GString, Error> {
+        // SAFETY: Since we said `false` for "allow_noent", this function must return a value
+        Ok(self.resolve_rev(refspec, false)?.unwrap())
+    }
+
     /// Write a content object from provided input.
     pub fn write_content<P: IsA<gio::InputStream>, Q: IsA<gio::Cancellable>>(
         &self,
@@ -381,4 +475,13 @@ impl Repo {
             );
         }))
     }
+
+    /// Load and parse directory metadata.
+    /// In particular, uid/gid/mode are stored in big-endian format; this function
+    /// converts them to host native endianness.
+    pub fn read_dirmeta(&self, checksum: &str) -> Result<crate::DirMetaParsed, glib::Error> {
+        let v = self.load_variant(crate::ObjectType::DirMeta, checksum)?;
+        // Safety: We know the variant type will match since we just passed it above
+        Ok(crate::DirMetaParsed::from_variant(&v).unwrap())
+    }
 }
