@@ -10,9 +10,13 @@ mod checkout_at;
 fn should_commit_content_to_repo_and_list_refs_again() {
     let test_repo = TestRepo::new();
 
+    assert!(test_repo.repo.require_rev("nosuchrev").is_err());
+
     let mtree = create_mtree(&test_repo.repo);
     let checksum = commit(&test_repo.repo, &mtree, "test");
 
+    assert_eq!(test_repo.repo.require_rev("test").unwrap(), checksum);
+
     let repo = ostree::Repo::new_for_path(test_repo.dir.path());
     repo.open(NONE_CANCELLABLE).expect("OSTree test_repo");
     let refs = repo
@@ -23,7 +27,31 @@ fn should_commit_content_to_repo_and_list_refs_again() {
 }
 
 #[test]
-fn should_traverse_commit() {
+#[cfg(feature = "cap-std-apis")]
+fn cap_std_commit() {
+    let test_repo = CapTestRepo::new();
+
+    assert!(test_repo.dir.exists("config"));
+    // Also test re-acquiring a new dfd
+    assert!(test_repo.repo.dfd_as_dir().unwrap().exists("config"));
+
+    assert!(test_repo.repo.require_rev("nosuchrev").is_err());
+
+    let mtree = create_mtree(&test_repo.repo);
+    let checksum = commit(&test_repo.repo, &mtree, "test");
+
+    assert_eq!(test_repo.repo.require_rev("test").unwrap(), checksum);
+
+    let repo2 = ostree::Repo::open_at_dir(&test_repo.dir, ".").unwrap();
+    let refs = repo2
+        .list_refs(None, NONE_CANCELLABLE)
+        .expect("failed to list refs");
+    assert_eq!(1, refs.len());
+    assert_eq!(checksum, refs["test"]);
+}
+
+#[test]
+fn repo_traverse_and_read() {
     let test_repo = TestRepo::new();
     let checksum = test_repo.test_commit("test");
 
@@ -54,6 +82,13 @@ fn should_traverse_commit() {
         ),
         objects
     );
+
+    let dirmeta = test_repo
+        .repo
+        .read_dirmeta("ad49a0f4e3bc165361b6d17e8a865d479b373ee67d89ac6f0ce871f27da1be6d")
+        .unwrap();
+    // Right now, the uid/gid are actually that of the test runner
+    assert_eq!(dirmeta.mode, 0o40750);
 }
 
 #[test]
